/* $Id: poisson.h,v 1.1 2001/10/25 15:55:45 knepley Exp $ */

/* Main include file for the Heat Equation test problem */

#ifndef __HEAT_H
#define __HEAT_H

#include "petscts.h"
#include "gsolver.h"

extern int HEAT_COOKIE;
extern int HEAT_ComputeField;

struct _HeatContext {
  PETSCHEADER(int)
  Grid                   grid;          /* The problem grid */

  MeshBoundary2D         boundaryCtx;   /* Describes the domain boundary */
  MeshGeometryContext    geometryCtx;   /* Describes the domain geometry */

  GTS                    ts;            /* The integrator context */
  GVec                   u;             /* The solution vector */
  GVec                   uExact;        /* The exact solution vector */

  int                    dim;           /* The problem dimension */
  PetscTruth             linear;        /* Using linear elements (as opposed to quadratic) */
  int                    refineStep;    /* The step to start refining the mesh */
  int                    numLoops;      /* The number of systems to solve */

  PetscScalar            decayExponent; /* The exponent k for the decay in our exact solution */

  GTSErrorMonitorCtx     monitorCtx;    /* Specialized monitor for grid vector integration */
  GVecErrorKSPMonitorCtx kspMonitorCtx; /* Specialized monitor for lienar solves */
};
typedef struct _HeatContext *HeatContext;

/* Creation functions */
EXTERN int HeatInitializePackage(char *);
EXTERN int HeatContextCreate(MPI_Comm, HeatContext *);
EXTERN int HeatContextSetup(HeatContext);
EXTERN int HeatContextCreateMeshBoundary(HeatContext);
EXTERN int HeatContextDestroyMeshBoundary(HeatContext);
EXTERN int HeatContextCreateMesh(HeatContext, Mesh *);
EXTERN int HeatContextCreateGrid(HeatContext);
EXTERN int HeatContextRefineGrid(HeatContext);
EXTERN int HeatCreateStructures(HeatContext);

/* Setup functions */
EXTERN int HeatSetupGrid(HeatContext);
EXTERN int HeatSetupRhsFunction(Grid, HeatContext);
EXTERN int HeatSetupBC(Grid, HeatContext);
EXTERN int HeatSetupKSP(KSP, HeatContext);
EXTERN int HeatSetupPC(PC, HeatContext);
EXTERN int HeatSetupStructures(HeatContext);

/* Cleanup functions */
EXTERN int HeatDestroyStructures(HeatContext);
EXTERN int HeatDestroy(HeatContext);
EXTERN int HeatContextDestroy(HeatContext);

/* Checking functions */
EXTERN int MatCheckSymmetry(Mat);
EXTERN int HeatCheckSolution(HeatContext, double, GVec, const char []);

/* Callback functions */
EXTERN int HeatSolutionFunction(Vec, void *);
EXTERN int SolutionFunction(int, int, double *, double *, double *, PetscScalar *, void*);
EXTERN int SolutionTimeDerivative(int, int, double *, double *, double *, PetscScalar *, void *);
EXTERN int RhsFunction(int, int, double *, double *, double *, PetscScalar *, void*);
EXTERN int HeatUpdateGTS(TS, double, double *);

/* Solver functions */
EXTERN int HeatSolve(HeatContext, double *);
EXTERN int HeatComputeField(HeatContext);

#endif /* __HEAT_H */
